@@ -11,7 +11,7 @@ use bytes::Bytes;
 use formdata::FormData;
 use http::method::Method;
 use hyper::body::HttpBody;
-use hyper::client::{connect::Connect, HttpConnector};
+use hyper::client::{connect::Connect, Client, HttpConnector};
 use hyper::header::{HeaderValue, CONTENT_TYPE};
 use hyper::{Body, HeaderMap, Request, Uri};
 #[cfg(feature = "for_serde")]
@@ -21,7 +21,7 @@ use serde_json;
 use url::Url;
 
 use super::simple_http;
-use super::simple_http::SimpleHTTP;
+use super::simple_http::{Interceptor, InterceptorFunc, SimpleHTTP};
 // use simple_http;
 
 /*
@@ -29,6 +29,15 @@ use super::simple_http::SimpleHTTP;
 */
 pub type PathParam = HashMap<String, String>;
 
+#[macro_export]
+macro_rules! path_param {
+    ($( $key: expr => $val: expr ),*) => {{
+         let mut map = hyper_api_service::simple_api::PathParam::new();
+         $( map.insert($key.into(), $val.into()); )*
+         map
+    }}
+}
+
 /*
 `CommonAPI` implements `make_api_response_only()`/`make_api_no_body()`/`make_api_has_body()`,
 for Retrofit-like usages.
@@ -50,6 +59,67 @@ impl<C, B> CommonAPI<C, B> {
     pub fn set_base_url(&self, url: Url) {
         self.simple_api.lock().unwrap().base_url = url;
     }
+    pub fn get_base_url_clone(&self) -> Url {
+        self.simple_api.lock().unwrap().base_url.clone()
+    }
+    pub fn set_default_header(&self, header_map: HeaderMap) {
+        self.simple_api.lock().unwrap().default_header = header_map;
+    }
+    pub fn get_default_header_clone(&self) -> HeaderMap {
+        self.simple_api.lock().unwrap().default_header.clone()
+    }
+    pub fn set_client(&self, client: Client<C, B>) {
+        self.simple_api.lock().unwrap().simple_http.client = client;
+    }
+    pub fn set_timeout_millisecond(&self, timeout_millisecond: u64) {
+        self.simple_api
+            .lock()
+            .unwrap()
+            .simple_http
+            .timeout_millisecond = timeout_millisecond;
+    }
+    pub fn get_timeout_millisecond(&self) -> u64 {
+        self.simple_api
+            .lock()
+            .unwrap()
+            .simple_http
+            .timeout_millisecond
+    }
+
+    pub fn add_interceptor(&mut self, interceptor: Arc<dyn Interceptor<B>>) {
+        self.simple_api
+            .lock()
+            .unwrap()
+            .simple_http
+            .add_interceptor(interceptor);
+    }
+    pub fn add_interceptor_front(&mut self, interceptor: Arc<dyn Interceptor<B>>) {
+        self.simple_api
+            .lock()
+            .unwrap()
+            .simple_http
+            .add_interceptor_front(interceptor);
+    }
+    pub fn delete_interceptor(&mut self, interceptor: Arc<dyn Interceptor<B>>) {
+        self.simple_api
+            .lock()
+            .unwrap()
+            .simple_http
+            .delete_interceptor(interceptor);
+    }
+}
+
+impl<C> CommonAPI<C, Body> {
+    pub fn add_interceptor_fn(
+        &mut self,
+        func: impl FnMut(&mut Request<Body>) -> StdResult<(), Box<dyn StdError>> + Send + Sync + 'static,
+    ) -> Arc<InterceptorFunc<Body>> {
+        self.simple_api
+            .lock()
+            .unwrap()
+            .simple_http
+            .add_interceptor_fn(func)
+    }
 }
 
 impl CommonAPI<HttpConnector, Body> {
